# Generated by roxygen2: do not edit by hand

export(allocate.basic)
export(allocate.distance)
export(allocate.diversity)
export(plot_dist)
export(select.distance)
export(select.diversity)
export(select.random)
import(mathjaxr)
importFrom(DiversityStats,mcintosh_diversity)
importFrom(DiversityStats,shannon)
importFrom(DiversityStats,simpson)
importFrom(MASS,isoMDS)
importFrom(Rdpack,reprompt)
importFrom(Rtsne,Rtsne)
importFrom(cluster,pam)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(grDevices,n2mfrow)
importFrom(graphics,par)
importFrom(graphics,title)
importFrom(igraph,E)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,mst)
importFrom(prospectr,duplex)
importFrom(prospectr,honigs)
importFrom(prospectr,kenStone)
importFrom(prospectr,naes)
importFrom(stats,as.dist)
importFrom(stats,cmdscale)
importFrom(stats,cutree)
importFrom(stats,hclust)
importFrom(stats,median)
importFrom(stats,setNames)
importFrom(vegan,betadisper)
