% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checks.sample.core.R
\name{checks.sample.core}
\alias{checks.sample.core}
\title{Common checks for all functions}
\usage{
checks.sample.core(
  data,
  names,
  size,
  group,
  qualitative = NULL,
  dist.mat = NULL,
  log.base = NULL,
  alloc,
  always.selected = NULL,
  mode = c("alloc", "sel")
)
}
\arguments{
\item{data}{The data as a data frame object. The data frame should possess
one row per individual and columns with the individual names and multiple
trait/character data.}

\item{names}{Name of column with the accession names as a character string.}

\item{size}{The desired core set size proportion.}

\item{group}{Name of column with the accession group/cluster names as a
character string.}

\item{qualitative}{Name of columns with the qualitative traits as a character
vector.}

\item{dist.mat}{A precomputed distance matrix of distance measures between the
accessions in \code{data}.}

\item{log.base}{The logarithm base to be used for logarithmic method of
sampling. Default is \code{exp(1)}.}

\item{alloc}{A named numeric vector specifying the number of entries to be
selected. Names should correspond to the levels of the "\code{"group"}
column, and values indicate the number of elements to be selected from each
level.}

\item{always.selected}{Names of accessions to be always included in the core
set as a character vector.}
}
\description{
Not exported. Strictly internal
}
\keyword{internal}
